@@ -30,9 +30,3 @@
 			 '(" " "Henkan_Mode" "down" "<IgnoreCase><Control>n"))
 (custom-set-default-key! 'generic-prev-candidate-key
 			 '("<Shift>Henkan_Mode" "up" "<IgnoreCase><Control>p"))
-(custom-set-default-key! 'anthy-transpose-as-hiragana-key '("F6" "Muhenkan"))
-(custom-set-default-key! 'anthy-transpose-as-katakana-key '("F7" "Muhenkan"))
-(custom-set-default-key! 'canna-transpose-as-hiragana-key '("F6" "Muhenkan"))
-(custom-set-default-key! 'canna-transpose-as-katakana-key '("F7" "Muhenkan"))
-(custom-set-default-key! 'mana-transpose-as-hiragana-key '("F6" "Muhenkan"))
-(custom-set-default-key! 'mana-transpose-as-katakana-key '("F7" "Muhenkan"))
