@@ -1,6 +1,6 @@
 ;;; custom.scm: Customization support
 ;;;
-;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -86,8 +86,10 @@
     (string? x)))
 
 (define custom-pathname?
-  (lambda (str)
-    (string? str)))
+  (lambda (str type)
+    (and (string? str)
+	 (symbol? type)
+	 (memq type '(regular-file directory)))))
 
 (define custom-valid-choice?
   (lambda arg
@@ -117,6 +119,10 @@
 			   (custom-exist? key 'key))))
 		key-repls))))
 
+(define custom-pathname-type
+  (lambda (custom-sym)
+    (car (custom-type-attrs custom-sym))))
+
 (define custom-expand-key-references
   (lambda (key)
     (cond
@@ -294,9 +300,7 @@
 ;; TODO: write test
 (define key-list-strip-shift
   (lambda (key-list)
-    (remove (lambda (key)
-	      (eq? key 'Shift_key))
-	    key-list)))
+    (delete 'Shift_key key-list eq?)))
 
 ;; TODO: write test
 (define key-list-strip-regular-shift
@@ -304,9 +308,7 @@
     (let* ((str (find string? key-list))
 	   (printable (string->printable-char str)))
       (if (char-graphic? printable)
-	  (remove (lambda (key)
-		    (eq? key 'Shift_key))
-		  key-list)
+	  (key-list-strip-shift key-list)
 	  key-list))))
 
 ;; TODO: write test
@@ -644,12 +646,13 @@
   (lambda (sym)
     (let* ((type (custom-type sym))
 	   (attrs (custom-type-attrs sym)))
-      (cond
-       ((or (eq? type 'choice)
-	    (eq? type 'ordered-list))
-	(map custom-choice-rec-sym attrs))
-       (else
-	attrs)))))
+      (case type
+	((choice ordered-list)
+	 (map custom-choice-rec-sym attrs))
+	((integer string)
+	 attrs)
+	(else
+	 ())))))
 
 ;; API
 (define custom-label
